// This file is generated by TypeBuilder_cpp.template.

// Copyright (c) 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "headless/lib/browser/protocol/dp_headless_experimental.h"

#include "headless/lib/browser/protocol/protocol.h"

#include "third_party/inspector_protocol/crdtp/cbor.h"
#include "third_party/inspector_protocol/crdtp/serializer_traits.h"

namespace headless {
namespace protocol {
namespace HeadlessExperimental {

// ------------- Enum values from types.

const char Metainfo::domainName[] = "HeadlessExperimental";
const char Metainfo::commandPrefix[] = "HeadlessExperimental.";
const char Metainfo::version[] = "1.3";

const char* ScreenshotParams::FormatEnum::Jpeg = "jpeg";
const char* ScreenshotParams::FormatEnum::Png = "png";

std::unique_ptr<ScreenshotParams> ScreenshotParams::fromValue(protocol::Value* value, ErrorSupport* errors)
{
    if (!value || value->type() != protocol::Value::TypeObject) {
        errors->addError("object expected");
        return nullptr;
    }

    std::unique_ptr<ScreenshotParams> result(new ScreenshotParams());
    protocol::DictionaryValue* object = DictionaryValue::cast(value);
    errors->push();
    protocol::Value* formatValue = object->get("format");
    if (formatValue) {
        errors->setName("format");
        result->m_format = ValueConversions<String>::fromValue(formatValue, errors);
    }
    protocol::Value* qualityValue = object->get("quality");
    if (qualityValue) {
        errors->setName("quality");
        result->m_quality = ValueConversions<int>::fromValue(qualityValue, errors);
    }
    errors->pop();
    if (errors->hasErrors())
        return nullptr;
    return result;
}

std::unique_ptr<protocol::DictionaryValue> ScreenshotParams::toValue() const
{
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    if (m_format.isJust())
        result->setValue("format", ValueConversions<String>::toValue(m_format.fromJust()));
    if (m_quality.isJust())
        result->setValue("quality", ValueConversions<int>::toValue(m_quality.fromJust()));
    return result;
}

void ScreenshotParams::AppendSerialized(std::vector<uint8_t>* out) const {
    crdtp::cbor::EnvelopeEncoder envelope_encoder;
    envelope_encoder.EncodeStart(out);
    out->push_back(crdtp::cbor::EncodeIndefiniteLengthMapStart());
      crdtp::SerializeField(crdtp::SpanFrom("format"), m_format, out);
      crdtp::SerializeField(crdtp::SpanFrom("quality"), m_quality, out);
    out->push_back(crdtp::cbor::EncodeStop());
    envelope_encoder.EncodeStop(out);
}

std::unique_ptr<ScreenshotParams> ScreenshotParams::clone() const
{
    ErrorSupport errors;
    return fromValue(toValue().get(), &errors);
}

std::unique_ptr<NeedsBeginFramesChangedNotification> NeedsBeginFramesChangedNotification::fromValue(protocol::Value* value, ErrorSupport* errors)
{
    if (!value || value->type() != protocol::Value::TypeObject) {
        errors->addError("object expected");
        return nullptr;
    }

    std::unique_ptr<NeedsBeginFramesChangedNotification> result(new NeedsBeginFramesChangedNotification());
    protocol::DictionaryValue* object = DictionaryValue::cast(value);
    errors->push();
    protocol::Value* needsBeginFramesValue = object->get("needsBeginFrames");
    errors->setName("needsBeginFrames");
    result->m_needsBeginFrames = ValueConversions<bool>::fromValue(needsBeginFramesValue, errors);
    errors->pop();
    if (errors->hasErrors())
        return nullptr;
    return result;
}

std::unique_ptr<protocol::DictionaryValue> NeedsBeginFramesChangedNotification::toValue() const
{
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    result->setValue("needsBeginFrames", ValueConversions<bool>::toValue(m_needsBeginFrames));
    return result;
}

void NeedsBeginFramesChangedNotification::AppendSerialized(std::vector<uint8_t>* out) const {
    crdtp::cbor::EnvelopeEncoder envelope_encoder;
    envelope_encoder.EncodeStart(out);
    out->push_back(crdtp::cbor::EncodeIndefiniteLengthMapStart());
      crdtp::SerializeField(crdtp::SpanFrom("needsBeginFrames"), m_needsBeginFrames, out);
    out->push_back(crdtp::cbor::EncodeStop());
    envelope_encoder.EncodeStop(out);
}

std::unique_ptr<NeedsBeginFramesChangedNotification> NeedsBeginFramesChangedNotification::clone() const
{
    ErrorSupport errors;
    return fromValue(toValue().get(), &errors);
}

// ------------- Enum values from params.


// ------------- Frontend notifications.

void Frontend::NeedsBeginFramesChanged(bool needsBeginFrames)
{
    if (!m_frontendChannel)
        return;
    std::unique_ptr<NeedsBeginFramesChangedNotification> messageData = NeedsBeginFramesChangedNotification::Create()
        .SetNeedsBeginFrames(needsBeginFrames)
        .Build();
    m_frontendChannel->sendProtocolNotification(InternalResponse::createNotification("HeadlessExperimental.needsBeginFramesChanged", std::move(messageData)));
}

void Frontend::flush()
{
    m_frontendChannel->flushProtocolNotifications();
}

void Frontend::sendRawCBORNotification(std::vector<uint8_t> notification)
{
    m_frontendChannel->sendProtocolNotification(InternalRawNotification::fromBinary(std::move(notification)));
}

// --------------------- Dispatcher.

class DispatcherImpl : public protocol::DispatcherBase {
public:
    DispatcherImpl(FrontendChannel* frontendChannel, Backend* backend)
        : DispatcherBase(frontendChannel)
        , m_backend(backend) {
        m_dispatchMap["HeadlessExperimental.beginFrame"] = &DispatcherImpl::beginFrame;
        m_dispatchMap["HeadlessExperimental.disable"] = &DispatcherImpl::disable;
        m_dispatchMap["HeadlessExperimental.enable"] = &DispatcherImpl::enable;
    }
    ~DispatcherImpl() override { }
    bool canDispatch(const String& method) override;
    void dispatch(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<protocol::DictionaryValue> messageObject) override;
    std::unordered_map<String, String>& redirects() { return m_redirects; }

protected:
    using CallHandler = void (DispatcherImpl::*)(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> messageObject, ErrorSupport* errors);
    using DispatchMap = std::unordered_map<String, CallHandler>;
    DispatchMap m_dispatchMap;
    std::unordered_map<String, String> m_redirects;

    void beginFrame(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);
    void disable(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);
    void enable(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport*);

    Backend* m_backend;
};

bool DispatcherImpl::canDispatch(const String& method) {
    return m_dispatchMap.find(method) != m_dispatchMap.end();
}

void DispatcherImpl::dispatch(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<protocol::DictionaryValue> messageObject)
{
    std::unordered_map<String, CallHandler>::iterator it = m_dispatchMap.find(method);
    DCHECK(it != m_dispatchMap.end());
    protocol::ErrorSupport errors;
    (this->*(it->second))(callId, method, message, std::move(messageObject), &errors);
}


class BeginFrameCallbackImpl : public Backend::BeginFrameCallback, public DispatcherBase::Callback {
public:
    BeginFrameCallbackImpl(std::unique_ptr<DispatcherBase::WeakPtr> backendImpl, int callId, const String& method, crdtp::span<uint8_t> message)
        : DispatcherBase::Callback(std::move(backendImpl), callId, method, message) { }

    void sendSuccess(bool hasDamage, Maybe<Binary> screenshotData) override
    {
        std::unique_ptr<protocol::DictionaryValue> resultObject = DictionaryValue::create();
        resultObject->setValue("hasDamage", ValueConversions<bool>::toValue(hasDamage));
        if (screenshotData.isJust())
            resultObject->setValue("screenshotData", ValueConversions<Binary>::toValue(screenshotData.fromJust()));
        sendIfActive(std::move(resultObject), DispatchResponse::OK());
    }

    void fallThrough() override
    {
        fallThroughIfActive();
    }

    void sendFailure(const DispatchResponse& response) override
    {
        DCHECK(response.status() == DispatchResponse::kError);
        sendIfActive(nullptr, response);
    }
};

void DispatcherImpl::beginFrame(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{
    // Prepare input parameters.
    protocol::DictionaryValue* object = DictionaryValue::cast(requestMessageObject->get("params"));
    errors->push();
    protocol::Value* frameTimeTicksValue = object ? object->get("frameTimeTicks") : nullptr;
    Maybe<double> in_frameTimeTicks;
    if (frameTimeTicksValue) {
        errors->setName("frameTimeTicks");
        in_frameTimeTicks = ValueConversions<double>::fromValue(frameTimeTicksValue, errors);
    }
    protocol::Value* intervalValue = object ? object->get("interval") : nullptr;
    Maybe<double> in_interval;
    if (intervalValue) {
        errors->setName("interval");
        in_interval = ValueConversions<double>::fromValue(intervalValue, errors);
    }
    protocol::Value* noDisplayUpdatesValue = object ? object->get("noDisplayUpdates") : nullptr;
    Maybe<bool> in_noDisplayUpdates;
    if (noDisplayUpdatesValue) {
        errors->setName("noDisplayUpdates");
        in_noDisplayUpdates = ValueConversions<bool>::fromValue(noDisplayUpdatesValue, errors);
    }
    protocol::Value* screenshotValue = object ? object->get("screenshot") : nullptr;
    Maybe<protocol::HeadlessExperimental::ScreenshotParams> in_screenshot;
    if (screenshotValue) {
        errors->setName("screenshot");
        in_screenshot = ValueConversions<protocol::HeadlessExperimental::ScreenshotParams>::fromValue(screenshotValue, errors);
    }
    errors->pop();
    if (errors->hasErrors()) {
        reportProtocolError(callId, DispatchResponse::kInvalidParams, kInvalidParamsString, errors);
        return;
    }

    std::unique_ptr<BeginFrameCallbackImpl> callback(new BeginFrameCallbackImpl(weakPtr(), callId, method, message));
    m_backend->BeginFrame(std::move(in_frameTimeTicks), std::move(in_interval), std::move(in_noDisplayUpdates), std::move(in_screenshot), std::move(callback));
    return;
}

void DispatcherImpl::disable(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{

    std::unique_ptr<DispatcherBase::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->Disable();
    if (response.status() == DispatchResponse::kFallThrough) {
        channel()->fallThrough(callId, method, message);
        return;
    }
    if (weak->get())
        weak->get()->sendResponse(callId, response);
    return;
}

void DispatcherImpl::enable(int callId, const String& method, crdtp::span<uint8_t> message, std::unique_ptr<DictionaryValue> requestMessageObject, ErrorSupport* errors)
{

    std::unique_ptr<DispatcherBase::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->Enable();
    if (response.status() == DispatchResponse::kFallThrough) {
        channel()->fallThrough(callId, method, message);
        return;
    }
    if (weak->get())
        weak->get()->sendResponse(callId, response);
    return;
}

// static
void Dispatcher::wire(UberDispatcher* uber, Backend* backend)
{
    std::unique_ptr<DispatcherImpl> dispatcher(new DispatcherImpl(uber->channel(), backend));
    uber->setupRedirects(dispatcher->redirects());
    uber->registerBackend("HeadlessExperimental", std::move(dispatcher));
}

} // HeadlessExperimental
} // namespace headless
} // namespace protocol
